import { Container } from 'inversify';
import { IViewport } from '../camera/ICameraService';
export declare type Point = [number, number];
export declare type Bounds = [[number, number], [number, number]];
export interface ILngLat {
    lng: number;
    lat: number;
}
export interface IPoint {
    x: number;
    y: number;
}
export interface IMercator {
    x: number;
    y: number;
    z: number;
}
export interface IStatusOptions {
    showIndoorMap: boolean;
    resizeEnable: boolean;
    dragEnable: boolean;
    keyboardEnable: boolean;
    doubleClickZoom: boolean;
    zoomEnable: boolean;
    rotateEnable: boolean;
}
export declare type MapStyle = string | {
    [key: string]: any;
};
export interface IMapWrapper {
    setContainer(container: Container, id: string | HTMLDivElement): void;
}
export interface IMapService<RawMap = {}> {
    map: RawMap;
    init(): void;
    destroy(): void;
    onCameraChanged(callback: (viewport: IViewport) => void): void;
    addMarkerContainer(): void;
    getMarkerContainer(): HTMLElement;
    on(type: string, handler: (...args: any[]) => void): void;
    off(type: string, handler: (...args: any[]) => void): void;
    once(type: string, handler: (...args: any[]) => void): void;
    getContainer(): HTMLElement | null;
    getSize(): [number, number];
    getMinZoom(): number;
    getMaxZoom(): number;
    getType(): string;
    getZoom(): number;
    getCenter(): ILngLat;
    getPitch(): number;
    getRotation(): number;
    getBounds(): Bounds;
    getMapContainer(): HTMLElement | null;
    getMapCanvasContainer(): HTMLElement;
    setRotation(rotation: number): void;
    zoomIn(option?: any, eventData?: any): void;
    zoomOut(option?: any, eventData?: any): void;
    panTo(p: Point): void;
    panBy(pixel: Point): void;
    fitBounds(bound: Bounds, fitBoundsOptions?: unknown): void;
    setZoomAndCenter(zoom: number, center: Point): void;
    setCenter(center: [number, number]): void;
    setPitch(pitch: number): void;
    setZoom(zoom: number): void;
    setMapStyle(style: any): void;
    setMapStatus(option: Partial<IStatusOptions>): void;
    pixelToLngLat(pixel: Point): ILngLat;
    lngLatToPixel(lnglat: Point): IPoint;
    containerToLngLat(pixel: Point): ILngLat;
    lngLatToContainer(lnglat: Point): IPoint;
    lngLatToMercator(lnglat: [number, number], altitude: number): IMercator;
    getModelMatrix(lnglat: [number, number], altitude: number, rotate: [number, number, number], scale: [number, number, number], origin: IMercator): number[];
    exportMap(type: 'jpg' | 'png'): string;
}
export declare const MapServiceEvent: string[];
/**
 * 地图初始化配置项
 */
export interface IMapConfig<RawMap = {}> {
    /**
     * 地图实例
     */
    mapInstance?: RawMap;
    /**
     * 高德地图API插件
     */
    plugin?: string[];
    /**
     * 容器 DOM id
     */
    id: string | HTMLDivElement;
    /**
     * 地图
     */
    token?: string;
    /**
     * 中心点
     */
    center?: [number, number];
    /**
     * 地图倾角
     */
    pitch?: number;
    /**
     * 地图旋转角度
     */
    rotation?: number;
    /**
     * 缩放等级
     */
    zoom?: number;
    /**
     * 底图样式
     */
    style?: MapStyle;
    /**
     * 最小缩放等级
     */
    minZoom?: number;
    /**
     * 最大缩放等级
     */
    maxZoom?: number;
    attributionControl?: boolean;
    [key: string]: any;
}
/**
 * 地图相机参数
 * @see
 */
export interface IMapCamera {
    aspect: number;
    fov: number;
    near: number;
    far: number;
    viewportWidth: number;
    viewportHeight: number;
    pitch: number;
    bearing: number;
    zoom: number;
    center: [number, number];
    cameraHeight: number;
    offsetOrigin: [number, number];
}
